from django.conf import settings

import abc
import importlib
import logging


class Authenticator:
    @abc.abstractmethod
    def is_authenticated(self, request):
        pass


class IsAuthenticated:

    def has_object_permission(self, request, view, object):
        return self.has_permission(request, view)

    def has_permission(self, request, view):
        if hasattr(settings, 'APP_AUTH'):
            auth_options = settings.APP_AUTH

            if not isinstance(auth_options, dict):
                return False

            if not 'authenticator' in auth_options:
                return False

            authenticator_class_name = auth_options['authenticator']

            module_name, class_name = authenticator_class_name.rsplit(".", 1)
            module = importlib.import_module(module_name)
            AuthenticatorClass = getattr(module, class_name)

            authenticator = AuthenticatorClass()
            return authenticator.is_authenticated(request)

        return True
