import datetime
import sys

from .aggregation_helpers import window_fn
from .data_provider import DataProvider


class Aggregator:

    def __init__(self):
        self.window_level = 0
        self.values = []

    def add(self, agregated_item):
        if agregated_item['cnt'] > 1:
            self.window_level = 1

            agregated_item['value'] = agregated_item['value'] / agregated_item['cnt']
            agregated_item['quality'] = agregated_item['quality'] / agregated_item['cnt']
        else:
            agregated_item['dt'] = agregated_item['dt_orig']

        del agregated_item['dt_orig']

        self.values.append(agregated_item)

    def result(self):
        return self.window_level, self.values

    def agregate(self, context, result):
        dt_start = context['dt_start']
        dt_end = context['dt_end']
        level = context['level']
        window_segment = context['window_segment']
        window_index_last = None
        agregated_item = None

        for item in result:
            window_index = window_fn(item[DataProvider.INDEX_DT], dt_start, window_segment)

            if agregated_item is None or window_index_last is None or window_index_last != window_index:
                if agregated_item is not None:
                    self.add(agregated_item)

                stamp = dt_start.timestamp() + window_segment * window_index
                window_index_last = window_index

                agregated_item = {
                    'dt': datetime.datetime.fromtimestamp(stamp),
                    'dt_orig': item[DataProvider.INDEX_DT],
                    'value': 0.0,
                    'value_min': +sys.float_info.max, # TODO MM
                    'value_max': -sys.float_info.max, # TODO MM
                    'quality': 0, # TODO MM

                    'level': window_index,
                    'cnt': 0,
                }

            # value
            value = float(item[DataProvider.INDEX_VALUE])

            # value min
            value_min = value
            if item[DataProvider.INDEX_VALUE_MIN] is not None:
                value_min = min(value, float(item[DataProvider.INDEX_VALUE_MIN]))

            # value max
            value_max = value
            if item[DataProvider.INDEX_VALUE_MAX] is not None:
                value_max = max(value, float(item[DataProvider.INDEX_VALUE_MAX]))

            # quality
            quality = float(item[DataProvider.INDEX_QUALITY])

            agregated_item['value'] = agregated_item['value'] + value
            agregated_item['quality'] = agregated_item['quality'] + quality
            agregated_item['value_min'] = min(agregated_item['value_min'], value_min)
            agregated_item['value_max'] = max(agregated_item['value_max'], value_max)
            agregated_item['cnt'] = agregated_item['cnt'] + 1

        if agregated_item is not None:
            self.add(agregated_item)
