import datetime
import logging

from .aggregation_helpers import window_fn
from .data_provider import DataProvider


class CumulativePreprocessor:

    def __init__(self):
        self.window_level = 0
        self.values = []
        self.value_last = None

    def add(self, agregated_item):
        if agregated_item['cnt'] > 1:
            self.window_level = 1

        agregated_item['quality'] = agregated_item['quality'] / agregated_item['cnt']

        value = agregated_item['value']

        if self.value_last is None:
            agregated_item['value'] = 0
            self.value_last = value
        else:
            diff = value - self.value_last
            if diff >= 0:
                agregated_item['value'] = diff
                self.value_last = max(self.value_last, value)
            else:
                agregated_item['value'] = 0
                agregated_item['quality'] = 0
                self.value_last = max(self.value_last, value)

        self.values.append(agregated_item)

    def add_empty(self, dt_start, window_segment, window_index):
        stamp = dt_start.timestamp() + window_segment * window_index

        agregated_item = {
            'dt': datetime.datetime.fromtimestamp(stamp),
            'value': 0.0,
            'value_min': 0.0, # TODO MM
            'value_max': 0.0, # TODO MM
            'quality': 1, # TODO MM

            'level': -1,
            'cnt': 0,
        }

        self.values.append(agregated_item)

    def result(self):
        return self.window_level, self.values

    def process(self, context, result):
        dt_start = context['dt_start']
        dt_end = context['dt_end']
        level = context['level']
        aggregation = context['aggregation']
        dt_diff = dt_end - dt_start
        dt_diff_seconds = dt_diff.total_seconds() # TODO MM move into helper function
        window_segment = aggregation * 60
        window_count = int(dt_diff_seconds / window_segment) # TODO MM move into helper function
        window_index_last = None
        agregated_item = None

        for item in result:
            window_index = window_fn(item[DataProvider.INDEX_DT], dt_start, window_segment)

            # value
            value = float(item[DataProvider.INDEX_VALUE])

            # value min
            value_min = value
            if item[DataProvider.INDEX_VALUE_MIN] is not None:
                value_min = min(value, float(item[DataProvider.INDEX_VALUE_MIN]))

            # value max
            value_max = value
            if item[DataProvider.INDEX_VALUE_MAX] is not None:
                value_max = max(value, float(item[DataProvider.INDEX_VALUE_MAX]))

            # quality
            quality = float(item[DataProvider.INDEX_QUALITY])

            if self.value_last is None:
                self.value_last = value_min

            if agregated_item is None or window_index_last is None or window_index_last != window_index:
                if agregated_item is not None:
                    self.add(agregated_item)

                if window_index_last is not None and window_index > window_index_last:
                    for index in range(window_index_last + 1, window_index):
                        self.add_empty(dt_start, window_segment, index)

                stamp = dt_start.timestamp() + window_segment * window_index
                window_index_last = window_index

                agregated_item = {
                    'dt': datetime.datetime.fromtimestamp(stamp),
                    'value': 0.0,
                    'value_min': 0.0, # TODO MM
                    'value_max': 0.0, # TODO MM
                    'quality': 0, # TODO MM

                    'level': window_index,
                    'cnt': 0,
                }

            agregated_item['value'] = max(agregated_item['value'], value_max)
            agregated_item['quality'] = agregated_item['quality'] + quality
            agregated_item['value_min'] = agregated_item['value']
            agregated_item['value_max'] = agregated_item['value']
            agregated_item['cnt'] = agregated_item['cnt'] + 1

        if agregated_item is not None:
            self.add(agregated_item)

        if window_index_last is not None and window_count > window_index_last:
            for index in range(window_index_last + 1, window_count):
                self.add_empty(dt_start, window_segment, index)
