from .core import ExportNoAggregation

from ..core.data_provider import DataProvider


class SignalAnalogNoAggregation(ExportNoAggregation):

    def create_header(self, sheet):
        headers = self.get_date_time_header()
        headers += [
            "value_avg",
            "value_min",
            "value_max",
            "quality",
        ]

        self.write_header(sheet, headers)

    def format_cells(self, sheet, signal):
        formats = self.get_date_time_header_format()
        formats += [
            self.create_number_format(signal),
            self.create_number_format(signal),
            self.create_number_format(signal),
            self.quality_format(),
        ]

        self.write_format(sheet, formats)

    def create_sheet(self, workbook, sheet, context, signal):
        self.create_header(sheet)

        items = self.generate_values(context, signal)
        index = 1

        for item in items:
            # stamp
            dt = item[DataProvider.INDEX_DT]

            # value
            value = float(item[DataProvider.INDEX_VALUE])

            # value min
            if item[DataProvider.INDEX_VALUE_MIN] is not None:
                value_min = float(item[DataProvider.INDEX_VALUE_MIN])
            else:
                value_min = None

            # value max
            if item[DataProvider.INDEX_VALUE_MAX] is not None:
                value_max = float(item[DataProvider.INDEX_VALUE_MAX])
            else:
                value_max = None

            # quality
            quality = float(item[DataProvider.INDEX_QUALITY])

            index = index + 1

            sheet.cell(column=1, row=index, value=dt)
            sheet.cell(column=2, row=index, value=dt)
            sheet.cell(column=3, row=index, value=value)
            sheet.cell(column=4, row=index, value=value_min)
            sheet.cell(column=5, row=index, value=value_max)
            sheet.cell(column=6, row=index, value=quality)

        self.format_cells(sheet, signal)
