from .core import ExportWithAggregation

from ..core.data_provider import DataProvider


class SignalAnalogWithAggregation(ExportWithAggregation):

    def create_header(self, sheet):
        headers = self.get_date_time_header()
        headers += [
            "value_avg",
            "value_min",
            "value_max",
            "quality",
            "samples",
        ]

        self.write_header(sheet, headers)

    def format_cells(self, sheet, signal):
        formats = self.get_date_time_header_format()
        formats += [
            self.create_number_format(signal),
            self.create_number_format(signal),
            self.create_number_format(signal),
            self.quality_format(),
            self.count_format(),
        ]

        self.write_format(sheet, formats)

    def create_sheet(self, workbook, sheet, context, signal):
        self.create_header(sheet)

        dt_start, window_segment = self.generate_date_time_columns(sheet, context)
        items = self.generate_values(context, signal)

        last_value = None
        agregated_item = None
        window_index_last = None

        for item in items:
            # stamp
            dt = item[DataProvider.INDEX_DT]

            # value
            value = float(item[DataProvider.INDEX_VALUE])

            # value min
            value_min = value
            if item[DataProvider.INDEX_VALUE_MIN] is not None:
                value_min = min(value, float(item[DataProvider.INDEX_VALUE_MIN]))

            # value max
            value_max = value
            if item[DataProvider.INDEX_VALUE_MAX] is not None:
                value_max = max(value, float(item[DataProvider.INDEX_VALUE_MAX]))

            # quality
            quality = float(item[DataProvider.INDEX_QUALITY])

            if last_value is None:
                last_value = value

            window_index = self.window_fn(dt, dt_start, window_segment)

            if agregated_item is None or window_index_last is None or window_index_last != window_index:
                if agregated_item is not None:
                    self.add(sheet, window_index_last, agregated_item)

                window_index_last = window_index

                agregated_item = {
                    "value_avg": 0,
                    "value_min": 0,
                    "value_max": 0,
                    "quality": 0,
                    "cnt": 0,
                }

            agregated_item["value_avg"] = agregated_item["value_avg"] + value
            agregated_item["value_min"] = min(agregated_item["value_min"], value_min)
            agregated_item["value_max"] = max(agregated_item["value_max"], value_max)
            agregated_item["quality"] = agregated_item["quality"] + quality
            agregated_item["cnt"] = agregated_item["cnt"] + 1

            last_value = value

        if agregated_item is not None:
            self.add(sheet, window_index_last, agregated_item)

        self.format_cells(sheet, signal)

    def add(self, sheet, index, item):
        if item["cnt"] > 0:
            item["quality"] = item["quality"] / item["cnt"]
            item["value_avg"] = item["value_avg"] / item["cnt"]

        row = self.get_row_index(index)

        sheet.cell(column=5, row=row, value=item["value_avg"])
        sheet.cell(column=6, row=row, value=item["value_min"])
        sheet.cell(column=7, row=row, value=item["value_max"])
        sheet.cell(column=8, row=row, value=item["quality"])
        sheet.cell(column=9, row=row, value=item["cnt"])
