from .core import ExportNoAggregation

from ..core.data_provider import DataProvider


class SignalBoolNoAggregation(ExportNoAggregation):

    def create_header(self, sheet):
        headers = self.get_date_time_header()
        headers += [
            "value",
            "quality",
        ]

        self.write_header(sheet, headers)

    def format_cells(self, sheet, signal):
        formats = self.get_date_time_header_format()
        formats += [
            self.create_number_format(signal),
            self.quality_format(),
        ]

        self.write_format(sheet, formats)

    def create_sheet(self, workbook, sheet, context, signal):
        self.create_header(sheet)

        items = self.generate_values(context, signal)
        index = 1

        last_value = None

        for item in items:
            # stamp
            dt = item[DataProvider.INDEX_DT]

            # value
            value = float(item[DataProvider.INDEX_VALUE])

            # quality
            quality = float(item[DataProvider.INDEX_QUALITY])

            if last_value is not None and last_value == value:
                continue

            last_value = value

            index = index + 1

            sheet.cell(column=1, row=index, value=dt)
            sheet.cell(column=2, row=index, value=dt)
            sheet.cell(column=3, row=index, value=value)
            sheet.cell(column=4, row=index, value=quality)

        self.format_cells(sheet, signal)
