from .core import ExportWithAggregation

from ..core.data_provider import DataProvider


class SignalBoolWithAggregation(ExportWithAggregation):

    def create_header(self, sheet):
        headers = self.get_date_time_header()
        headers += [
            "on_duration",
            "on_cnt",
            "off_duration",
            "off_cnt",
            "quality",
        ]

        self.write_header(sheet, headers)

    def format_cells(self, sheet, signal):
        formats = self.get_date_time_header_format()
        formats += [
            self.count_format(),
            self.count_format(),
            self.count_format(),
            self.count_format(),
            self.quality_format(),
        ]

        self.write_format(sheet, formats)

    def create_sheet(self, workbook, sheet, context, signal):
        self.create_header(sheet)

        dt_start, window_segment = self.generate_date_time_columns(sheet, context)
        items = self.generate_values(context, signal)

        last_value = None
        last_value_stamp = None
        agregated_item = None
        window_index_last = None

        for item in items:
            # stamp
            dt = item[DataProvider.INDEX_DT]

            # value
            value = float(item[DataProvider.INDEX_VALUE])

            # quality
            quality = float(item[DataProvider.INDEX_QUALITY])

            if last_value is None:
                last_value = value

            if last_value_stamp is None:
                last_value_stamp = dt_start.timestamp()

            window_index = self.window_fn(dt, dt_start, window_segment)

            if agregated_item is None or window_index_last is None or window_index_last != window_index:
                if agregated_item is not None:
                    diff = dt_start.timestamp() + window_segment * window_index - last_value_stamp
                    if last_value:
                        agregated_item["on_duration"] = agregated_item["on_duration"] + diff
                    else:
                        agregated_item["off_duration"] = agregated_item["off_duration"] + diff

                    self.add(sheet, window_index_last, agregated_item)

                window_index_last = window_index
                last_value_stamp = dt_start.timestamp() + window_segment * window_index

                agregated_item = {
                    "on_duration": 0,
                    "on_cnt": 0,
                    "off_duration": 0,
                    "off_cnt": 0,
                    "quality": 0,
                    "cnt": 0,
                }

            if last_value != value:
                if value:
                    agregated_item["on_cnt"] = agregated_item["on_cnt"] + 1

                    diff = dt.timestamp() - last_value_stamp
                    last_value_stamp = dt.timestamp()
                    agregated_item["off_duration"] = agregated_item["off_duration"] + diff
                else:
                    agregated_item["off_cnt"] = agregated_item["off_cnt"] + 1

                    diff = dt.timestamp() - last_value_stamp
                    last_value_stamp = dt.timestamp()
                    agregated_item["on_duration"] = agregated_item["on_duration"] + diff

            agregated_item["quality"] = agregated_item["quality"] + quality
            agregated_item["cnt"] = agregated_item["cnt"] + 1

            last_value = value

        if agregated_item is not None:
            diff = dt_start.timestamp() + window_segment * window_index - last_value_stamp
            if last_value:
                agregated_item["on_duration"] = agregated_item["on_duration"] + diff
            else:
                agregated_item["off_duration"] = agregated_item["off_duration"] + diff

            self.add(sheet, window_index_last, agregated_item)

        self.format_cells(sheet, signal)

    def add(self, sheet, index, item):
        if item["cnt"] > 0:
            item["quality"] = item["quality"] / item["cnt"]

        row = self.get_row_index(index)

        sheet.cell(column=5, row=row, value=item["on_duration"])
        sheet.cell(column=6, row=row, value=item["on_cnt"])
        sheet.cell(column=7, row=row, value=item["off_duration"])
        sheet.cell(column=8, row=row, value=item["off_cnt"])
        sheet.cell(column=9, row=row, value=item["quality"])
