from .core import ExportWithAggregation

from ..core.data_provider import DataProvider


class SignalCumulativeWithAggregation(ExportWithAggregation):

    def create_header(self, sheet):
        headers = self.get_date_time_header()
        headers += [
            "time_value",
            "value_last",
            "quality",
            "value_delta",
        ]

        self.write_header(sheet, headers)

    def format_cells(self, sheet, signal):
        formats = self.get_date_time_header_format()
        formats += [
            self.time_format(),
            self.create_number_format(signal),
            self.quality_format(),
            self.create_number_format(signal),
        ]

        self.write_format(sheet, formats)

    def create_sheet(self, workbook, sheet, context, signal):
        self.create_header(sheet)

        dt_start, window_segment = self.generate_date_time_columns(sheet, context)
        items = self.generate_values(context, signal)

        last_value = None
        agregated_item = None
        window_index_last = None

        for item in items:
            # stamp
            dt = item[DataProvider.INDEX_DT]

            # value
            value = float(item[DataProvider.INDEX_VALUE])

            # quality
            quality = float(item[DataProvider.INDEX_QUALITY])

            if last_value is None:
                last_value = value

            window_index = self.window_fn(dt, dt_start, window_segment)

            if agregated_item is None or window_index_last is None or window_index_last != window_index:
                if agregated_item is not None:
                    self.add(sheet, window_index_last, agregated_item)

                window_index_last = window_index

                agregated_item = {
                    "time_value": None,
                    "value_last": 0,
                    "quality": 0,
                    "value_delta": 0,
                    "cnt": 0,
                }

            agregated_item["time_value"] = dt
            agregated_item["value_last"] = value
            agregated_item["value_delta"] = value - last_value  # TODO MM
            agregated_item["quality"] = agregated_item["quality"] + quality
            agregated_item["cnt"] = agregated_item["cnt"] + 1

            last_value = value

        if agregated_item is not None:
            self.add(sheet, window_index_last, agregated_item)

        self.format_cells(sheet, signal)

    def add(self, sheet, index, item):
        if item["cnt"] > 0:
            item["quality"] = item["quality"] / item["cnt"]

        row = self.get_row_index(index)

        sheet.cell(column=5, row=row, value=item["time_value"])
        sheet.cell(column=6, row=row, value=item["value_last"])
        sheet.cell(column=7, row=row, value=item["quality"])
        sheet.cell(column=8, row=row, value=item["value_delta"])
