from openpyxl.styles import Alignment

from .core import ExportWithAggregation

from ..core.data_provider import DataProvider


class SignalsAnalogWithAggregation(ExportWithAggregation):

    def get_row_index(self, index):
        return index + 3

    def create_header(self, sheet, signals):
        headers = self.get_date_time_header()

        for signal in signals:
            headers += [
                "value_avg",
                "value_min",
                "value_max",
                "quality",
                "samples",
            ]

        self.write_header(sheet, headers, row_index=2)

    def format_cells(self, sheet, signals):
        formats = self.get_date_time_header_format()
        for signal in signals:
            formats += [
                self.create_number_format(signal),
                self.create_number_format(signal),
                self.create_number_format(signal),
                self.quality_format(),
                self.count_format(),
            ]

        self.write_format(sheet, formats)

    def create_sheet(self, workbook, sheet, context, signals):
        self.create_header(sheet, signals)

        dt_start, window_segment = self.generate_date_time_columns(sheet, context)
        signal_index = -1

        for signal in signals:
            items = self.generate_values(context, signal)
            signal_index = signal_index + 1

            last_value = None
            agregated_item = None
            window_index_last = None

            for item in items:
                # stamp
                dt = item[DataProvider.INDEX_DT]

                # value
                value = float(item[DataProvider.INDEX_VALUE])

                # value min
                value_min = value
                if item[DataProvider.INDEX_VALUE_MIN] is not None:
                    value_min = min(value, float(item[DataProvider.INDEX_VALUE_MIN]))

                # value max
                value_max = value
                if item[DataProvider.INDEX_VALUE_MAX] is not None:
                    value_max = max(value, float(item[DataProvider.INDEX_VALUE_MAX]))

                # quality
                quality = float(item[DataProvider.INDEX_QUALITY])

                if last_value is None:
                    last_value = value

                window_index = self.window_fn(dt, dt_start, window_segment)

                if agregated_item is None or window_index_last is None or window_index_last != window_index:
                    if agregated_item is not None:
                        self.add(sheet, window_index_last, agregated_item, signal_index)

                    window_index_last = window_index

                    agregated_item = {
                        "value_avg": 0,
                        "value_min": 0,
                        "value_max": 0,
                        "quality": 0,
                        "cnt": 0,
                    }

                agregated_item["value_avg"] = agregated_item["value_avg"] + value
                agregated_item["value_min"] = min(agregated_item["value_min"], value_min)
                agregated_item["value_max"] = max(agregated_item["value_max"], value_max)
                agregated_item["quality"] = agregated_item["quality"] + quality
                agregated_item["cnt"] = agregated_item["cnt"] + 1

                last_value = value

            if agregated_item is not None:
                self.add(sheet, window_index_last, agregated_item, signal_index)

        self.format_cells(sheet, signals)

        signal_index = -1
        for signal in signals:
            signal_index = signal_index + 1

            cell = sheet.cell(column=5 + signal_index*5, row=1, value=signal.id)
            cell.number_format = "0"
            cell.alignment = Alignment(horizontal='center')

            sheet.merge_cells(start_row=1, end_row=1, start_column=(5 + signal_index*5), end_column=(9 + signal_index*5))

    def add(self, sheet, index, item, signal_index):
        if item["cnt"] > 0:
            item["quality"] = item["quality"] / item["cnt"]
            item["value_avg"] = item["value_avg"] / item["cnt"]

        row = self.get_row_index(index)

        sheet.cell(column=5 + signal_index*5, row=row, value=item["value_avg"])
        sheet.cell(column=6 + signal_index*5, row=row, value=item["value_min"])
        sheet.cell(column=7 + signal_index*5, row=row, value=item["value_max"])
        sheet.cell(column=8 + signal_index*5, row=row, value=item["quality"])
        sheet.cell(column=9 + signal_index*5, row=row, value=item["cnt"])
