from .core import ExportNoAggregation

from ..core.data_provider import DataProvider


class SignalsBoolNoAggregation(ExportNoAggregation):

    def create_header(self, sheet):
        headers = self.get_date_time_header()
        headers += [
            "signal_id",
            "value",
            "quality",
        ]

        self.write_header(sheet, headers)

    def format_cells(self, sheet):
        formats = self.get_date_time_header_format()
        formats += [
            "0",
            "0.00",
            self.quality_format(),
        ]

        self.write_format(sheet, formats)

    def create_sheet(self, workbook, sheet, context, signals):
        self.create_header(sheet)

        items = self.generate_values_signals(context, signals)
        index = 1

        last_value = {}
        for signal in signals:
            last_value[signal.id] = None

        for item in items:
            # signal
            signal_id = item[DataProvider.INDEX_SIGNAL_ID]

            # stamp
            dt = item[DataProvider.INDEX_DT]

            # value
            value = float(item[DataProvider.INDEX_VALUE])

            # quality
            quality = float(item[DataProvider.INDEX_QUALITY])

            if last_value[signal_id] is not None and last_value[signal_id] == value:
                continue

            last_value[signal_id] = value

            index = index + 1

            sheet.cell(column=1, row=index, value=dt)
            sheet.cell(column=2, row=index, value=dt)
            sheet.cell(column=3, row=index, value=signal_id)
            sheet.cell(column=4, row=index, value=value)
            sheet.cell(column=5, row=index, value=quality)

        self.format_cells(sheet)
