from api_base.core.data_provider import DataProvider as BaseDataProvider
from api_base.core.dynamic_import import dynamic_import

from api_database.models import get_data_model


Data, Signal, Datatbl = dynamic_import(".models", ["Data", "Signal", "Datatbl"])


class DataProvider(BaseDataProvider):

    def load(self, context, signal):
        dt_start = context['dt_start']
        dt_end = context['dt_end']

        # TODO MM
        datatbl = Datatbl.objects \
            .filter(tblspace=signal.tblspace) \
            .filter(date_from__lte=dt_end) \
            .filter(date_to__gte=dt_start) \
            .order_by('date_from')

        for datatbl_item in datatbl:
            model = get_data_model(Data, datatbl_item.name)
            items = model.objects \
                .filter(signal=signal) \
                .filter(dt__gte=dt_start) \
                .filter(dt__lte=dt_end) \
                .values_list('dt', 'value', 'value_min', 'value_max', 'quality', 'signal_id') \
                .order_by('dt')

            for item in items:
                yield item

    def load_multi(self, context, signals):
        dt_start = context['dt_start']
        dt_end = context['dt_end']

        tblspace_ids = []
        signal_ids = []
        for signal in signals:
            tblspace_ids.append(signal.tblspace_id)
            signal_ids.append(signal.id)

        # TODO MM
        datatbl = Datatbl.objects \
            .filter(tblspace__in=tblspace_ids) \
            .filter(date_from__lte=dt_end) \
            .filter(date_to__gte=dt_start) \
            .order_by('date_from')

        for datatbl_item in datatbl:
            model = get_data_model(Data, datatbl_item.name)
            items = model.objects \
                .filter(signal__in=signal_ids) \
                .filter(dt__gte=dt_start) \
                .filter(dt__lte=dt_end) \
                .values_list('dt', 'value', 'value_min', 'value_max', 'quality', 'signal_id') \
                .order_by('dt')

            for item in items:
                yield item
