from django.db import connections

from api_base.core.data_provider import DataProvider as BaseDataProvider
from api_base.core.dynamic_import import dynamic_import


Signal, Datatbl = dynamic_import(".models", ["Signal", "Datatbl"])


class DataProvider(BaseDataProvider):

    def load(self, context, signal):
        dt_start = context['dt_start']
        dt_end = context['dt_end']

        # TODO MM
        datatbl = Datatbl.objects \
            .filter(tblspace=signal.tblspace) \
            .filter(date_from__lte=dt_end) \
            .filter(date_to__gte=dt_start) \
            .order_by('date_from')

        with connections['default'].cursor() as cursor:
            for datatbl_item in datatbl:
                query = "\
                    SELECT \
                        dt, \
                        value, \
                        value_min, \
                        value_max, \
                        quality, \
                        signal_id \
                    FROM %s \
                    WHERE signal_id = %s \
                    AND dt >= %s \
                    AND dt <= %s \
                    ORDER BY dt ASC \
                " % (
                    datatbl_item.name,
                    signal.pk,
                    "to_date('%s', 'YYYY-MM-DD hh24:mi')" % (dt_start.strftime("%Y-%m-%d %H:%M")),
                    "to_date('%s', 'YYYY-MM-DD hh24:mi')" % (dt_end.strftime("%Y-%m-%d %H:%M")),
                )

                cursor.execute(query)

                rows = cursor.fetchall()
                for row in rows:
                    yield row

    def load_multi(self, context, signals):
        dt_start = context['dt_start']
        dt_end = context['dt_end']

        tblspace_ids = []
        signal_ids = []
        for signal in signals:
            tblspace_ids.append(signal.tblspace_id)
            signal_ids.append(signal.id)

        # TODO MM
        datatbl = Datatbl.objects \
            .filter(tblspace__in=tblspace_ids) \
            .filter(date_from__lte=dt_end) \
            .filter(date_to__gte=dt_start) \
            .order_by('date_from')

        with connections['default'].cursor() as cursor:
            for datatbl_item in datatbl:
                query = "\
                    SELECT \
                        dt, \
                        value, \
                        value_min, \
                        value_max, \
                        quality, \
                        signal_id \
                    FROM %s \
                    WHERE signal_id IN (%s) \
                    AND dt >= %s \
                    AND dt <= %s \
                    ORDER BY dt ASC \
                " % (
                    datatbl_item.name,
                    ",".join(str(v) for v in signal_ids),
                    "to_date('%s', 'YYYY-MM-DD hh24:mi')" % (dt_start.strftime("%Y-%m-%d %H:%M")),
                    "to_date('%s', 'YYYY-MM-DD hh24:mi')" % (dt_end.strftime("%Y-%m-%d %H:%M")),
                )

                cursor.execute(query)

                rows = cursor.fetchall()
                for row in rows:
                    yield row
