from django.test import TestCase

import json
import os
import logging


class ViewsTestCase(TestCase):

    def write(self, content, filename):
        return
        
        dirname = os.path.dirname(__file__)
        filename = os.path.join(dirname, filename)

        with open(filename, 'w') as f_out:
            f_out.write(content.decode("utf-8"))

    def get_expected_response(self, filename):
        dirname = os.path.dirname(__file__)
        filename = os.path.join(dirname, filename)
        
        with open(filename) as f:
            return f.read()

    def assertJsonEquals(self, value, expected):
        value_json = json.loads(value)
        expected_json = json.loads(expected)

        self.assertEqual(value_json, expected_json)

    def test_get_signals(self):
        response = self.client.get('/arcview/api/v1/signal/')
        
        self.write(response.content, "fixtures/signal.json")
        self.assertJsonEquals(response.content.decode("utf-8"), self.get_expected_response("fixtures/signal.json"))
        self.assertEqual(response.status_code, 200)

    def test_get_signal(self):
        response = self.client.get('/arcview/api/v1/signal/2/')
        
        self.write(response.content, "fixtures/signal_2.json")
        self.assertJsonEquals(response.content.decode("utf-8"), self.get_expected_response("fixtures/signal_2.json"))
        self.assertEqual(response.status_code, 200)

    def test_get_signal_group(self):
        response = self.client.get('/arcview/api/v1/signal/2/group/')
        
        self.write(response.content, "fixtures/signal_2_group.json")
        self.assertJsonEquals(response.content.decode("utf-8"), self.get_expected_response("fixtures/signal_2_group.json"))
        self.assertEqual(response.status_code, 200)

    def test_get_signal_data_type_auto(self):
        response = self.client.get('/arcview/api/v1/signal/2/data/?dt_start=2021-11-01-00-00-00&dt_end=2021-11-30-00-00-00&level=1024')
        
        self.write(response.content, "fixtures/signal_2_data_auto.json")
        self.assertJsonEquals(response.content.decode("utf-8"), self.get_expected_response("fixtures/signal_2_data_auto.json"))
        self.assertEqual(response.status_code, 200)

    def test_get_signal_data_type_analog(self):
        response = self.client.get('/arcview/api/v1/signal/2/data/as/analog/?dt_start=2021-11-01-00-00-00&dt_end=2021-11-30-00-00-00&level=1024')
        
        self.write(response.content, "fixtures/signal_2_data_analog.json")
        self.assertJsonEquals(response.content.decode("utf-8"), self.get_expected_response("fixtures/signal_2_data_analog.json"))
        self.assertEqual(response.status_code, 200)

    def test_get_signal_data_type_cumulative(self):
        response = self.client.get('/arcview/api/v1/signal/2/data/as/cumulative/?dt_start=2021-11-01-00-00-00&dt_end=2021-11-30-00-00-00&level=1024')
        
        self.write(response.content, "fixtures/signal_2_data_cumulative.json")
        self.assertJsonEquals(response.content.decode("utf-8"), self.get_expected_response("fixtures/signal_2_data_cumulative.json"))
        self.assertEqual(response.status_code, 200)

    def test_get_signal_data_type_bool(self):
        response = self.client.get('/arcview/api/v1/signal/2/data/as/bool/?dt_start=2021-11-01-00-00-00&dt_end=2021-11-30-00-00-00&level=1024')
        
        self.write(response.content, "fixtures/signal_2_data_bool.json")
        self.assertJsonEquals(response.content.decode("utf-8"), self.get_expected_response("fixtures/signal_2_data_bool.json"))
        self.assertEqual(response.status_code, 200)

    def test_get_groups(self):
        response = self.client.get('/arcview/api/v1/group/')
        
        self.write(response.content, "fixtures/group.json")
        self.assertJsonEquals(response.content.decode("utf-8"), self.get_expected_response("fixtures/group.json"))
        self.assertEqual(response.status_code, 200)

    def test_get_group(self):
        response = self.client.get('/arcview/api/v1/group/1/')
        
        self.write(response.content, "fixtures/group_1.json")
        self.assertJsonEquals(response.content.decode("utf-8"), self.get_expected_response("fixtures/group_1.json"))
        self.assertEqual(response.status_code, 200)

    def test_get_option(self):
        response = self.client.get('/arcview/api/v1/option/')
        
        self.write(response.content, "fixtures/option.json")
        self.assertJsonEquals(response.content.decode("utf-8"), self.get_expected_response("fixtures/option.json"))
        self.assertEqual(response.status_code, 200)
