from ..core.buffer import Buffer
from .constants import ARS_SUBTYPE_SHIFT, ARC_CLASS_MASK, ARC_ALARM_FIRST, \
    ARC_ALARM_LAST, ARC_GROUP_FIRST, ARC_GROUP_LAST, ARC_ALARM_ACKN, \
    ARC_LONG_ALARM, CAL_ORIG_YEAR, ALARM_SIZES, SOF_TCNI, ARC_BOOL_GROUP, GROUP_ITEM_SIZES, \
    ARS_SUBTYPE_MASK, ARC_CLASS_LAST, ARS_LAST, OSTM_NANOSEC_PER_DAY, CLASS_NAMES, \
    SYSTEM_SUBTYPES, BOOL_SUBTYPES, VAL_SUBTYPES_SHORT, VAL_SUBTYPES_LONG, STR_AL_SUBTYPES


def XFAILED(value):
    return value <= -100  # TODO MM


def ARCODE(alclass, alsubtype):
    return alclass | (alsubtype << ARS_SUBTYPE_SHIFT)


def ARCLASS(alcode):
    return (alcode & ARC_CLASS_MASK)


def ARSUBTYPE(alcode):
    return (alcode & ARS_SUBTYPE_MASK) >> ARS_SUBTYPE_SHIFT


def XBYTEARR2XWORD(buffer: Buffer):
    return \
        (buffer.byte_at(0) << 8) | \
        (buffer.byte_at(1))


def XBYTEARR2XDWORD(buffer: Buffer):
    return \
        (buffer.byte_at(0) << 24) | \
        (buffer.byte_at(1) << 16) | \
        (buffer.byte_at(2) << 8) | \
        (buffer.byte_at(3))


def valid_item_code(code):
    return \
        ((code >= ARC_ALARM_FIRST) and (code <= ARC_ALARM_LAST)) or \
        ((code >= ARC_GROUP_FIRST) and (code <= ARC_GROUP_LAST)) or \
        (code == ARC_ALARM_ACKN)


def get_alarm_size(class_name):
    if class_name == ARC_ALARM_ACKN:
        return ALARM_SIZES[ARC_LONG_ALARM]

    # XASSERT((class_name >= ARC_ALARM_FIRST) and (class_name <= ARC_ALARM_LAST))

    return ALARM_SIZES[class_name]


def get_group_size(class_name, count):
    # XASSERT((class_name >= ARC_GROUP_FIRST) && (class_name <= ARC_GROUP_LAST))

    return SOF_TCNI + (
        (32 if (count == 0) else (count + 7) / 8)
        if class_name == ARC_BOOL_GROUP
        else (GROUP_ITEM_SIZES[class_name - ARC_GROUP_FIRST]) * count)


def code_date(date):
    return date * OSTM_NANOSEC_PER_DAY


def code_time(ticks_hi, ticks_lo):
    ticks = (ticks_hi << 32) | (ticks_lo)

    # XASSERT(ticks < (XLARGE)OSTM_NANOSEC_PER_DAY)

    return ticks


def is_leap_year(year):
    return (year % 4 == 0) and ((year % 100 != 0) or (year % 400 == 0))


day_sum_tab = [0, 31, 59, 90, 120, 151, 181, 212, 243, 273, 304, 334]


def set_days_from_origin(days_from_orig):
    days_from_orig = days_from_orig + 1
    diff_year = int(days_from_orig/366)
    year = CAL_ORIG_YEAR + diff_year
    is_leap = is_leap_year(year)
    days_from_orig = days_from_orig % 366 + diff_year - (diff_year + 3)/4 + (year > 2000)*((year - 2001)/100)

    while (days_from_orig > 365 + is_leap):
        year = year + 1
        days_from_orig -= 365 + is_leap
        is_leap = is_leap_year(year)

    month = 12
    while (day_sum_tab[month - 1] + (is_leap and (month > 2)) >= days_from_orig):
        month = month - 1

    day = int(days_from_orig) - day_sum_tab[month - 1] - (is_leap and (month > 2))

    return year, month, day


def decode_date(ticks):
    # XASSERT((year != NULL) && (month != NULL) && (day != NULL))

    return set_days_from_origin(int(ticks / OSTM_NANOSEC_PER_DAY))


def decode_time(ticks):
    # XASSERT(ticks < (XLARGE)OSTM_NANOSEC_PER_DAY)
    # XASSERT((hour != NULL) && (min != NULL) && (sec != NULL) && (nano_sec != NULL))

    hour = int(ticks / 3600000000000)
    ticks %= 3600000000000
    min = int(ticks / 60000000000)
    ticks %= 60000000000
    sec = int(ticks / 1000000000)
    ticks %= 1000000000
    nano_sec = ticks

    return hour, min, sec, nano_sec


def decode_timestamp(dest_DT, timestamp):
    # XASSERT(dest_DT != NULL)
    # XASSERT(pSrcTStamp != NULL)

    dest_DT.year, dest_DT.month, dest_DT.day = decode_date(timestamp.ticks)
    dest_DT.hour, dest_DT.min, dest_DT.sec, dest_DT.nano_sec = decode_time(timestamp.ticks % OSTM_NANOSEC_PER_DAY)

    # /* Consistency check (_XDEBUG only) */
    # XASSERT(pSrcTStamp.ticks % OSTM_NANOSEC_PER_DAY ==
    #         ((dest_DT.hour*_ULL(60) + dest_DT.min)*_ULL(60) +
    #         dest_DT.sec)*_ULL(1000000000) + dest_DT.nano_sec)


def get_arc_class_name(code):
    code = ARCLASS(code)

    if code > ARC_CLASS_LAST:
        code = ARC_CLASS_LAST + 1

    return CLASS_NAMES[code]


def get_arc_system_subtypes(sub_type):
    # XASSERT(sizeof(system_subtypes) / sizeof(system_subtypes[0]) == ARS_SYSTEM_LAST + 2)
    # check if system_subtypes[] is consistant with archive defitition

    if sub_type >= 4:
        sub_type = 3

    return SYSTEM_SUBTYPES[sub_type]


def get_arc_bool_subtypes(sub_type):
    if sub_type > ARS_LAST:
        sub_type = ARS_LAST + 1

    return BOOL_SUBTYPES[sub_type]


def get_arc_val_subtypes(sub_type):
    if sub_type > ARS_LAST:
        sub_type = ARS_LAST + 1

    return VAL_SUBTYPES_SHORT[sub_type]


def get_str_al_subtypes(sub_type):
    if sub_type > ARS_LAST:
        sub_type = ARS_LAST + 1

    return STR_AL_SUBTYPES[sub_type]


def get_arc_ack_subtypes(sub_type):
    if sub_type > ARS_LAST:
        sub_type = ARS_LAST + 1

    return VAL_SUBTYPES_LONG[sub_type]
