from ctypes import Union, Array, c_bool, c_byte, c_double, c_float, c_int16, c_int32, c_int64, \
    c_uint16, c_uint32, c_uint8

from .constants import AGS_BOOL_ARR_SIZE, AGS_BYTE_ARR_SIZE, AGS_DOUBLE_ARR_SIZE, AGS_DWORD_ARR_SIZE, \
    AGS_ERROR_ARR_SIZE, AGS_FLOAT_ARR_SIZE, AGS_LARGE_ARR_SIZE, AGS_LONG_ARR_SIZE, AGS_SHORT_ARR_SIZE, \
    AGS_TIME_ARR_SIZE, AGS_WORD_ARR_SIZE


class AL_UNION(Union):
    _fields_ = \
        ("f_bool", c_bool), \
        ("f_byte", c_uint8), \
        ("f_short", c_int16), \
        ("f_long", c_int32), \
        ("f_word", c_uint16), \
        ("f_dword", c_uint32), \
        ("f_float", c_float), \
        ("f_double", c_double), \
        ("f_time", c_double), \
        ("f_large", c_int64), \
        ("f_error", c_int16)


class BoolArray(Array):
    _type_ = c_bool
    _length_ = AGS_BOOL_ARR_SIZE


class ByteArray(Array):
    _type_ = c_byte
    _length_ = AGS_BYTE_ARR_SIZE


class ShortArray(Array):
    _type_ = c_int16
    _length_ = AGS_SHORT_ARR_SIZE


class LongArray(Array):
    _type_ = c_int32
    _length_ = AGS_LONG_ARR_SIZE


class WordArray(Array):
    _type_ = c_uint16
    _length_ = AGS_WORD_ARR_SIZE


class DWordArray(Array):
    _type_ = c_uint32
    _length_ = AGS_DWORD_ARR_SIZE


class LargeArray(Array):
    _type_ = c_int64
    _length_ = AGS_LARGE_ARR_SIZE


class FloatArray(Array):
    _type_ = c_float
    _length_ = AGS_FLOAT_ARR_SIZE


class DoubleArray(Array):
    _type_ = c_double
    _length_ = AGS_DOUBLE_ARR_SIZE


class TimeArray(Array):
    _type_ = c_double
    _length_ = AGS_TIME_ARR_SIZE


class ErrorArray(Array):
    _type_ = c_int16
    _length_ = AGS_ERROR_ARR_SIZE


class AG_UNION(Union):
    _fields_ = \
        ("f_bool_array", BoolArray), \
        ("f_byte_array", ByteArray), \
        ("f_short_array", ShortArray), \
        ("f_long_array", LongArray), \
        ("f_word_array", WordArray), \
        ("f_dword_array", DWordArray), \
        ("f_large_array", LargeArray), \
        ("f_float_array", FloatArray), \
        ("f_double_array", DoubleArray), \
        ("f_time_array", TimeArray), \
        ("f_error_array", ErrorArray)


class AU(Union):
    _fields_ = \
        ("alu", AL_UNION), \
        ("agu", AG_UNION)


class AR_COMMON_ITEM:
    def __init__(self):
        self.ticks_hi = None
        self.ticks_lo = None
        self.code = None
        self.lev_cnt = None
        self.ID = None
        self.length = None
        self.au = AU()


class ACI_WITH_DATE:
    def __init__(self):
        self.aci = AR_COMMON_ITEM()
        self.date = None
        self.marked = None


class ADATETIME:
    def __init__(self):
        self.year = 0
        self.month = 0
        self.day = 0
        self.hour = 0
        self.min = 0
        self.sec = 0
        self.nano_sec = 0


class GTSTAMP:
    def __init__(self):
        self.ticks = 0
