from os import path, walk

import re


class Explorer:
    def explore(self, archive_root_path, context=None):
        if not path.exists(archive_root_path):
            raise Exception('Archive directory does not exist!')

        pattern = re.compile(
            r"""
            (?P<year>\d{4}?)
            \-
            (?P<month>\d{2}?)
            \-
            (?P<day>\d{2}?)
            \_
            (?P<name>.*?)
            \.rea
            """, re.VERBOSE)

        result = []

        for root, subdirs, files in walk(archive_root_path):
            for filename in files:
                file_path = path.join(root, filename)

                if filename.endswith(".rea"):
                    match = pattern.match(filename)
                    stamp = match.group("year") + "-" + match.group("month") + "-" + match.group("day")

                    if self.filter_date(stamp, context):
                        result.append({
                            "file_path": file_path,
                            "stamp": stamp,
                            "year": match.group("year"),
                            "month": match.group("month"),
                            "day": match.group("day"),
                            "name": match.group("name"),
                        })

        return result

    def filter_date(self, stamp, context):
        if context and "dt_start" in context:
            dt = context["dt_start"].strftime('%Y-%m-%d')
            if stamp < dt:
                return False

        if context and "dt_end" in context:
            dt = context["dt_end"].strftime('%Y-%m-%d')
            if stamp > dt:
                return False

        return True
