from ..core.constants import ARC_ALARM_ACKN, ARC_ALARM_LAST, ARC_BOOL_ALARM, ARC_BOOL_GROUP, ARC_BYTE_ALARM, ARC_BYTE_GROUP, \
    ARC_DOUBLE_ALARM, \
    ARC_DOUBLE_GROUP, ARC_DWORD_ALARM, ARC_DWORD_GROUP, ARC_ERROR_ALARM, ARC_FLOAT_ALARM, ARC_FLOAT_GROUP, ARC_LARGE_ALARM, \
    ARC_LARGE_GROUP, ARC_LONG_ALARM, ARC_LONG_GROUP, ARC_SHORT_ALARM, ARC_SHORT_GROUP, ARC_STRING_ALARM, ARC_SYSTEM_ALARM, \
    ARC_TIME_ALARM, ARC_WORD_ALARM, ARC_WORD_GROUP
from ..core.helpers import ARCLASS


class ArchiveRow:
    ALCI_SYSTEM = 0
    ALCI_ALARM = 1
    ALCI_FINISHED = 2
    ALCI_ACKNED = 3
    ALCI_GROUP = 4
    ALCIL_ERROR = 5
    ALCIL_WARNING = 6
    ALCIL_INFO = 7
    ALCIL_VERB = 8
    ALCIL_READ = 9
    ALCIL_WRITE = 10

    TYPE_UNDEFINED = 1
    TYPE_BOOL = 2
    TYPE_INT = 3
    TYPE_DOUBLE = 4
    TYPE_STRING = 5

    def __init__(self, acid):
        self.acid = acid
        self.date = None
        self.time = None
        self.class_name = None
        self.type = None
        self.level = None
        self.ID = None
        self.col_count = None

    def get_JSON_data_type(self):
        code = ARCLASS(self.acid.aci.code)

        if ((code <= ARC_ALARM_LAST) or (code == ARC_ALARM_ACKN)):
            if code == ARC_SYSTEM_ALARM:
                return self.TYPE_UNDEFINED

            if code == ARC_BOOL_ALARM:
                return self.TYPE_UNDEFINED

            if code == ARC_BYTE_ALARM:
                return self.TYPE_INT

            if code == ARC_SHORT_ALARM:
                return self.TYPE_INT

            if code == ARC_LONG_ALARM:
                return self.TYPE_INT

            if code == ARC_WORD_ALARM:
                return self.TYPE_INT

            if code == ARC_DWORD_ALARM:
                return self.TYPE_INT

            if code == ARC_FLOAT_ALARM:
                return self.TYPE_DOUBLE

            if code == ARC_DOUBLE_ALARM:
                return self.TYPE_DOUBLE

            if code == ARC_TIME_ALARM:
                return self.TYPE_DOUBLE

            if code == ARC_LARGE_ALARM:
                return self.TYPE_DOUBLE

            if code == ARC_ERROR_ALARM:
                return self.TYPE_INT

            if code == ARC_STRING_ALARM:
                if self.acid.aci.au.alu.xString is not None:
                    return self.TYPE_UNDEFINED

                return self.TYPE_STRING

            if code == ARC_ALARM_ACKN:  # acknowledge has an integer parameter - process like long-alarm
                return self.TYPE_INT

            return self.TYPE_STRING

        if code == ARC_BOOL_GROUP:
            return self.TYPE_BOOL

        if code == ARC_BYTE_GROUP:
            return self.TYPE_INT

        if code == ARC_SHORT_GROUP:
            return self.TYPE_INT

        if code == ARC_LONG_GROUP:
            return self.TYPE_INT

        if code == ARC_WORD_GROUP:
            return self.TYPE_INT

        if code == ARC_DWORD_GROUP:
            return self.TYPE_INT

        if code == ARC_LARGE_GROUP:
            return self.TYPE_DOUBLE

        if code == ARC_FLOAT_GROUP:
            return self.TYPE_DOUBLE

        if code == ARC_DOUBLE_GROUP:
            return self.TYPE_DOUBLE

        return self.TYPE_STRING

    def get_bool(self, index):
        code = ARCLASS(self.acid.aci.code)

        if code == ARC_BOOL_GROUP:
            return (self.acid.aci.au.agu.f_bool_array[index / 8] & (1 << index % 8)) != 0

        return False

    def get_int(self, index):
        code = ARCLASS(self.acid.aci.code)

        if ((code <= ARC_ALARM_LAST) or (code == ARC_ALARM_ACKN)):
            if code == ARC_BYTE_ALARM:
                return self.acid.aci.au.alu.f_byte
            if code == ARC_SHORT_ALARM:
                return self.acid.aci.au.alu.f_short
            if code == ARC_LONG_ALARM:
                return self.acid.aci.au.alu.f_long
            if code == ARC_WORD_ALARM:
                return self.acid.aci.au.alu.f_word
            if code == ARC_DWORD_ALARM:
                return self.acid.aci.au.alu.f_dword
            if code == ARC_ERROR_ALARM:
                return self.acid.aci.au.alu.f_error
            if code == ARC_ALARM_ACKN:  # acknowledge has an integer parameter - process like long-alarm
                return self.acid.aci.au.alu.f_long

        else:
            if code == ARC_BYTE_GROUP:
                return self.acid.aci.au.agu.f_byte_array[index]
            if code == ARC_SHORT_GROUP:
                return self.acid.aci.au.agu.f_short_array[index]
            if code == ARC_LONG_GROUP:
                return self.acid.aci.au.agu.f_long_array[index]
            if code == ARC_WORD_GROUP:
                return self.acid.aci.au.agu.f_word_array[index]
            if code == ARC_DWORD_GROUP:
                return self.acid.aci.au.agu.f_dword_array[index]

        return -1

    def get_double(self, index):
        code = ARCLASS(self.acid.aci.code)

        if ((code <= ARC_ALARM_LAST) or (code == ARC_ALARM_ACKN)):
            if code == ARC_FLOAT_ALARM:
                return self.acid.aci.au.alu.f_float
            if code == ARC_DOUBLE_ALARM:
                return self.acid.aci.au.alu.f_double
            if code == ARC_TIME_ALARM:
                return self.acid.aci.au.alu.f_time
            if code == ARC_LARGE_ALARM:
                return self.acid.aci.au.alu.f_large

        else:
            if code == ARC_LARGE_GROUP:
                return self.acid.aci.au.agu.f_large_array[index]
            if code == ARC_FLOAT_GROUP:
                return self.acid.aci.au.agu.f_float_array[index]
            if code == ARC_DOUBLE_GROUP:
                return self.acid.aci.au.agu.f_double_array[index]

        return -1

    def get_string(self):
        code = ARCLASS(self.acid.aci.code)

        if ((code <= ARC_ALARM_LAST) or (code == ARC_ALARM_ACKN)):
            if code == ARC_STRING_ALARM:
                if self.acid.aci.au.alu.xString is not None:
                    return self.acid.aci.au.alu.xString

                else:
                    return ""

            else:
                return "UNKNOWN ALARM!"

        return None
