from ..core.constants import ARC_SYSTEM_ALARM
from ..core.helpers import ARCLASS
from ..writer.archive_row import ArchiveRow
from ..parser.parser import Parser
from ..explorer.explorer import Explorer
from ..core.structs import ACI_WITH_DATE
from ..writer.writer import BaseWriter


class SignalTableCreator(BaseWriter):

    INACTIVITY_LIMIT = 1000

    def __init__(self, path):
        self.signals = {}
        self.path = path
        self.counter = 0

    def load(self):
        explorer = Explorer()
        archives = explorer.explore(self.path, {})
        archives.sort(key=lambda x: x["stamp"], reverse=True)

        files = []
        for archive in archives:
            files.append(archive['file_path'])

        parser = Parser()
        parser.export_files(self, files)

        values = self.signals.values()
        final = []

        for signals in values:
            final = final + list(signals.values())

        return final

    def serialize_signal_id(self, signal_id, signal_index):
        return signal_id * 1024 + signal_index  # TODO MM 1024 as global constant

    def export_value(self, acid: ACI_WITH_DATE):
        row = self._parseArchiveItem(acid)

        if row.ID and row.ID not in self.signals:
            self.signals[row.ID] = {}
            self.counter = 0

            if row.col_count > 1:
                for index in range(0, row.col_count):
                    self.signals[row.ID][index] = {
                        "id": self.serialize_signal_id(row.ID, index),
                        "name": "signal " + str(row.ID) + ".index[" + str(index) + "]",
                        "description": None,
                        "type": 'ANALOG',
                        "unit": None,
                        "decimals": None,
                        "rex_value_path": None,
                    }
            else:
                self.signals[row.ID][0] = {
                    "id": row.ID,
                    "name": "signal " + str(row.ID),
                    "description": None,
                    "type": 'ANALOG',
                    "unit": None,
                    "decimals": None,
                    "rex_value_path": None,
                }
        else:
            self.counter = self.counter + 1

        if self.counter < self.INACTIVITY_LIMIT:
            return True
        else:
            return False

    def _parseArchiveItem(self, value):
        archiveItem = ArchiveRow(value)
        archiveItem.col_count = value.aci.lev_cnt

        code = ARCLASS(value.aci.code)

        if code != ARC_SYSTEM_ALARM:
            archiveItem.ID = value.aci.ID

        return archiveItem
