from ..core.constants import ARC_SYSTEM_ALARM
from ..core.helpers import ARCLASS, code_date, code_time, decode_timestamp
from ..writer.archive_row import ArchiveRow
from ..parser.parser import Parser
from ..explorer.explorer import Explorer
from ..core.structs import ACI_WITH_DATE, ADATETIME, GTSTAMP
from ..writer.writer import BaseWriter

import datetime


class SignalValueGenerator(BaseWriter):

    def __init__(self, path):
        self.context = None
        self.signal = None
        self.signal_id = None
        self.signal_index = None
        self.path = path

    def load(self, context, signal):
        self.context = context
        self.signal = signal
        self.signal_id = self.parse_signal_id(signal.id)
        self.signal_index = self.parse_signal_index(signal.id)

        explorer = Explorer()
        archives = explorer.explore(self.path, self.context)
        archives.sort(key=lambda x: x["stamp"])

        files = []
        for archive in archives:
            files.append(archive['file_path'])

        parser = Parser()
        yield from parser.export_files_as_generator(self, files)

    def load_multi(self, context, signals):
        # TODO MM
        raise Exception('Not implemented')

    def parse_signal_id(self, signal_id):
        return int(signal_id / 1024)  # TODO MM 1024 as global constant

    def parse_signal_index(self, signal_id):
        return int(signal_id % 1024)  # TODO MM 1024 as global constant

    def export_value(self, acid: ACI_WITH_DATE):
        if self.signal_id != acid.aci.ID:
            return
            yield

        row = self._parseArchiveItem(acid)
        dt_string = row.date + " " + row.time
        dt = datetime.datetime.strptime(dt_string, '%Y-%m-%d %H:%M:%S.%f')
        datatype = row.get_JSON_data_type()
        index = row.col_count - self.signal_index - 1 # TODO MM byte reverse order

        if datatype == ArchiveRow.TYPE_STRING:
            yield [
                dt,
                row.get_string(),
                None,
                None,
                1.0,
            ]

        elif datatype != ArchiveRow.TYPE_UNDEFINED:
            if datatype == ArchiveRow.TYPE_BOOL:
                yield [
                    dt,
                    "on" if row.get_bool(index) else "off",
                    None,
                    None,
                    1.0,
                ]

            elif datatype == ArchiveRow.TYPE_INT:
                yield [
                    dt,
                    str(row.get_int(index)),
                    None,
                    None,
                    1.0,
                ]

            elif datatype == ArchiveRow.TYPE_DOUBLE:
                yield [
                    dt,
                    str(row.get_double(index)),
                    None,
                    None,
                    1.0,
                ]

            else:
                raise Exception('TODO')

    def _parseArchiveItem(self, value):
        archiveItem = ArchiveRow(value)
        archiveItem.col_count = value.aci.lev_cnt

        dt = ADATETIME()
        code = ARCLASS(value.aci.code)

        # Date and Time
        timeStamp = GTSTAMP()
        timeStamp.ticks = code_date(value.date) + code_time(value.aci.ticks_hi, value.aci.ticks_lo)
        decode_timestamp(dt, timeStamp)

        if timeStamp.ticks == 0:
            # system time not initialized yet
            archiveItem.date = "---"
            archiveItem.time = "---"

        else:
            archiveItem.date = str(dt.year) + "-" + str(dt.month) + "-" + str(dt.day if dt.day > 0 else 1) # TODO MM
            archiveItem.time = str(dt.hour) + ":" + str(dt.min) + ":" + str(dt.sec) + "." + str(int(dt.nano_sec/1000))

        # ID
        if code != ARC_SYSTEM_ALARM:
            archiveItem.ID = value.aci.ID

        return archiveItem
