from django.conf import settings

from api_base.core.data_provider import DataProvider as BaseDataProvider

from ..arc_reader.writer.signal_value_generator import SignalValueGenerator

import datetime


class DataProvider(BaseDataProvider):

    def load(self, context, signal):
        result = []

        if settings.ARCHIVES_CPYTHON_LIBRARY is True:
            try:
                import arc_reader

                signals = []
                signals.append(signal.id)

                result1 = arc_reader.read(
                    settings.ARCHIVES_ROOT_DIR,
                    context["dt_start"].strftime("%Y-%m-%d %H:%M:%S"),
                    context["dt_end"].strftime("%Y-%m-%d %H:%M:%S"),
                    signals
                )

                result = []
                for row in result1:
                    dt = datetime.datetime(*map(int, [
                        row[0],
                        row[1],
                        row[2],
                        row[3],
                        row[4],
                        row[5],
                        row[6],
                    ]))

                    result.append([
                        dt,
                        row[7],
                        row[8],
                        row[9],
                        row[10],
                        row[11],
                    ])
            except Exception as e:
                provider = SignalValueGenerator(settings.ARCHIVES_ROOT_DIR)
                result = provider.load(context, signal)
        else:
            provider = SignalValueGenerator(settings.ARCHIVES_ROOT_DIR)
            result = provider.load(context, signal)

        return result

    def load_multi(self, context, signals):
        result = []

        signals = [signal.id for signal in signals]

        if settings.ARCHIVES_CPYTHON_LIBRARY is True:
            try:
                import arc_reader

                result1 = arc_reader.read(
                    settings.ARCHIVES_ROOT_DIR,
                    context["dt_start"].strftime("%Y-%m-%d %H:%M:%S"),
                    context["dt_end"].strftime("%Y-%m-%d %H:%M:%S"),
                    signals
                )

                result = []
                for row in result1:
                    dt = datetime.datetime(*map(int, [
                        row[0],
                        row[1],
                        row[2],
                        row[3],
                        row[4],
                        row[5],
                        row[6],
                    ]))

                    result.append([
                        dt,
                        row[7],
                        row[8],
                        row[9],
                        row[10],
                        row[11],
                    ])
            except Exception as e:
                provider = SignalValueGenerator(settings.ARCHIVES_ROOT_DIR)
                result = provider.load_multi(context, signals)
        else:
            provider = SignalValueGenerator(settings.ARCHIVES_ROOT_DIR)
            result = provider.load_multi(context, signals)

        return result
