from django.core.management.base import BaseCommand
from django.conf import settings

from ...arc_reader.core.structs import ACI_WITH_DATE
from ...arc_reader.explorer.explorer import Explorer
from ...arc_reader.writer.writer import BaseWriter
from ...arc_reader.parser.parser import Parser

import csv


###############################################################################
# WRITER
###############################################################################
class SignalCsvWriter(BaseWriter):

    def __init__(self, file_name):
        self.signals = {}
        self.file_name = file_name
        self.handle = None
        self.writer = None

    def open(self):
        self.handle = open(self.file_name, 'w', newline='')

        self.writer = csv.writer(self.handle)
        self.writer.writerow([
            "id",
            "name",
            "description",
            "comment",
            "type",
            "unit",
            "decimals",
            "rexcore",
            "rex_value_path",
            "rex_trend_path",
            "range_min",
            "range_max",
        ])

    def export_value(self, acid: ACI_WITH_DATE):
        id = acid.aci.ID

        if id and id not in self.signals:
            self.signals[id] = {
                "id": id,
                "name": "signal " + str(acid.aci.ID),
            }

        return True

    def close(self):
        for key in self.signals:
            signal = self.signals[key]
            self.writer.writerow(signal.values())

        self.handle.close()


###############################################################################
# MAIN
###############################################################################
class Command(BaseCommand):
    def handle(self, *args, **options):
        path = settings.ARCHIVES_ROOT_DIR

        explorer = Explorer()
        archives = explorer.explore(path)

        files = []
        for archive in archives:
            files.append(archive['file_path'])

        writer = SignalCsvWriter(settings.ARCHIVES_SIGNAL_META_FILE_PATH)
        writer.open()

        parser = Parser()
        parser.export_files(writer, files)

        writer.close()
