import csv
import logging

from os import path


class Signal:
    def __init__(self):
        self.id = None
        self.name = None
        self.description = None
        self.type = "ANALOG"
        self.unit = None
        self.decimals = 0
        self.rexcore_id = ""
        self.rex_trend_path = ""
        self.rex_value_path = ""
        self.range_min = None
        self.range_max = None
        self.groups = []


class SignalExplorer:
    def __init__(self, file_name):
        self.signals = []
        self.file_name = file_name
        self.handle = None
        self.reader = None

    def open(self):
        if not path.exists(self.file_name):
            # raise Exception("File " + self.file_name + " does not exist!")
            return False

        self.handle = open(self.file_name, 'r', newline='')

        self.reader = csv.reader(self.handle)

        return True

    def read(self):
        next(self.reader)

        for row in self.reader:
            signal = Signal()
            signal.id = int(row[0]) if len(row) > 0 and row[0] else None
            signal.name = row[1] if len(row) > 1 and row[1] else None
            signal.description = row[2] if len(row) > 2 and row[2] else None
            signal.type = row[3] if len(row) > 3 and row[3] else None
            signal.unit = row[4] if len(row) > 4 and row[4] else None
            signal.decimals = row[5] if len(row) > 5 and row[5] else 0
            signal.rexcore_id = int(row[6]) if len(row) > 6 and row[6] and int(row[6]) and int(row[6]) > 0 else None
            signal.rex_value_path = row[7] if len(row) > 7 and row[7] else None

            self.signals.append(signal)

    def close(self):
        self.handle.close()

    def get_signals(self):
        if self.open():
            self.read()
            self.close()

        return self.signals

    def get_signal(self, pk):
        signals = self.get_signals()

        for signal in signals:
            if signal.id == pk:
                return signal

        return None


class SignalExplorerWriter:
    def __init__(self, file_name):
        self.signals = []
        self.file_name = file_name
        self.handle = None
        self.writer = None

    def open(self):
        self.handle = open(self.file_name, 'w', newline='')

        self.writer = csv.writer(self.handle)

    def update(self, signals):
        self.open()
        self.write(signals)
        self.close()

    def close(self):
        self.handle.close()

    def write(self, signals):
        self.writer.writerow([
            "id",
            "name",
            "description",
            "type",
            "unit",
            "decimals",
            "rexcore_id",
            "rex_value_path",
        ])

        for signal in signals:
            id = signal['id'] if 'id' in signal and signal['id'] is not None else None
            if not id:
                continue

            self.writer.writerow([
                id,
                signal['name'] if 'name' in signal and signal['name'] is not None else '',
                signal['description'] if 'description' in signal and signal['description'] is not None else '',
                signal['type'] if 'type' in signal and signal['type'] is not None else '',
                signal['unit'] if 'unit' in signal and signal['unit'] is not None else '',
                signal['decimals'] if 'decimals' in signal and signal['decimals'] is not None else '',
                signal['rexcore_id'] if 'rexcore_id' in signal and signal['rexcore_id'] is not None else 0,
                signal['rex_value_path'] if 'rex_value_path' in signal and signal['rex_value_path'] is not None else '',
            ])
