# -*- coding: utf-8 -*-
from django.core.management.base import BaseCommand, CommandError
from django.conf import settings

from os import path, remove


class Command(BaseCommand):

    def add_arguments(self, parser):
        parser.add_argument(
            '--force',
            type=bool,
            required=False,
            help='Replace existing configuration'
        )

        parser.add_argument(
            '--database',
            type=str,
            required=True,
            help='Database Type'
        )

        parser.add_argument(
            '--host',
            type=str,
            required=True,
            help='Host'
        )

        parser.add_argument(
            '--user',
            type=str,
            required=True,
            help='User'
        )

        parser.add_argument(
            '--password',
            type=str,
            required=True,
            help='Password'
        )

        parser.add_argument(
            '--name',
            type=str,
            required=True,
            help='Name'
        )

        parser.add_argument(
            '--port',
            type=str,
            required=True,
            help='Port'
        )

        parser.add_argument(
            '--schema',
            type=str,
            required=True,
            help='Schema'
        )

        parser.add_argument(
            '--route_api',
            type=str,
            required=False,
            help='Route'
        )
    
    def handle(self, *args, **options):
        self._init_settings_file(options)

        return str(True)

    def _init_settings_file(self, options):
        type = options['database']
        type = "db_" + type  # TODO MM

        force = options['force']

        default_settings_path = path.join(settings.BASE_DIR, "_config", type, "settings.py")
        if path.exists(default_settings_path):
            if not force:
                raise CommandError('Application is already initialized, force flag is not enabled!')
            else:
                remove(default_settings_path)

        template_settings_path = path.join(settings.BASE_DIR, "_config", type, "settings_template.py")
        if not path.exists(template_settings_path):
            raise CommandError('Template settings does not exists!')

        with open(template_settings_path, 'r') as file:
            content = file.read()

            if 'route_api' in options and options['route_api']:
                route_api = options['route_api']
                lines = content.splitlines()
                content = ""

                for line in lines:
                    if "# Application definition" in line:
                        content = content + "\n\n" + "# Overrides by bootstrap"
                        content = content + "\n"
                        content = content + "APP_URL_API_ROUTE = \"" + route_api + "\""
                        content = content + "\n"
                        content = content + "DEBUG = False"
                        content = content + "\n"
                        content = content + "APP_OPTION = {\n"
                        content = content + "    'rexcore/edit': True,\n"
                        content = content + "    'signal/edit': True,\n"
                        content = content + "}"
                        content = content + "\n"
                        content = content + "\n"
                        content = content + "\n"

                    content = content + line
                    content = content + "\n"

            value = options["host"] if "host" in options and options["host"] else ""
            content = content.replace('#HOST#', value)

            value = options["port"] if "port" in options and options["port"] else ""
            content = content.replace('#PORT#', value)

            value = options["user"] if "user" in options and options["user"] else ""
            content = content.replace('#USER#', value)

            value = options["password"] if "password" in options and options["password"] else ""
            content = content.replace('#PASSWORD#', value)

            value = options["name"] if "name" in options and options["name"] else ""
            content = content.replace('#NAME#', value)

            value = options["schema"] if "schema" in options and options["schema"] else ""
            content = content.replace('#SCHEMA#', value)

            with open(default_settings_path, "w") as configuration:
                configuration.write(content)
