# -*- coding: utf-8 -*-
from django.core.management.base import BaseCommand
from django.db import connections
from django.conf import settings

import importlib
import uuid
import getpass


class Command(BaseCommand):

    def add_arguments(self, parser):
        parser.add_argument(
            '--host',
            type=str,
            required=True,
            help='Host'
        )

        parser.add_argument(
            '--user',
            type=str,
            required=True,
            help='User'
        )

        parser.add_argument(
            '--password',
            type=str,
            required=True,
            help='Password'
        )

        parser.add_argument(
            '--name',
            type=str,
            required=True,
            help='Name'
        )

        parser.add_argument(
            '--port',
            type=str,
            required=True,
            help='Port'
        )

        parser.add_argument(
            '--schema',
            type=str,
            required=True,
            help='Schema'
        )

    def handle(self, *args, **options):
        db_conn = self._get_database_connection(options)

        try:
            db_conn.cursor()
        except Exception:
            connected = False
        else:
            connected = True

        return str(connected)

    def _get_database_connection(self, options):
        database_id = str(uuid.uuid4())

        newDatabase = {}
        newDatabase["id"] = database_id
        newDatabase['ENGINE'] = 'django.db.backends.postgresql_psycopg2'
        newDatabase['NAME'] = options['name']
        newDatabase['USER'] = options['user']
        newDatabase['PASSWORD'] = options['password']
        newDatabase['HOST'] = options['host']
        newDatabase['PORT'] = options['port']
        connections.databases[database_id] = newDatabase

        return connections[database_id]
