# -*- coding: utf-8 -*-
from django.core.management.base import BaseCommand, CommandError
from django.db import connections
from django.conf import settings

from os import path, listdir

import calendar
import datetime
import importlib
import uuid


class Command(BaseCommand):

    def add_arguments(self, parser):
        parser.add_argument(
            '--database',
            type=str,
            required=True,
            help='Database Type'
        )

        parser.add_argument(
            '--date',
            type=str,
            required=True,
            help='Date'
        )

    def handle(self, *args, **options):
        db_conn = self._get_db_conn(options)
        self._generate_schema(db_conn, options)

        return str(True)

    def _get_db_conn(self, options):
        database = options['database']

        impmodule = importlib.import_module("_config.db_" + database + ".settings")
        DATABASES = impmodule.DATABASES

        connections.databases[database] = DATABASES['default']
        db_conn = connections[database]

        return db_conn

    def _generate_schema(self, db_conn, options):
        date = options['date']  # TODO MM validate it

        if date == "-":
            return self._generate_persistent_table(db_conn)
        else:
            return self._generate_date_table(db_conn, date)

    def _generate_persistent_table(self, db_conn):
        table_name = "ALL"
        date_from = "1970-01-01"  # TODO MM
        date_to = "2030-12-31"  # TODO MM

        return self._generate_data_model(db_conn, table_name, date_from, date_to)
        
    def _generate_date_table(self, db_conn, date):
        date_full = date + "01"
        
        try:
            date_object = datetime.datetime.strptime(date_full, '%Y%m%d')
        except ValueError:
            raise ValueError("Incorrect data format, should be YYYY-MM-DD")

        table_name = date
        date_range = calendar.monthrange(date_object.year, date_object.month)
        date_from = str(date_object.year) + "-" + str(date_object.month) + "-01"
        date_to = str(date_object.year) + "-" + str(date_object.month) + "-" + str(date_range[1])

        return self._generate_data_model(db_conn, table_name, date_from, date_to)

    def _generate_data_model(self, db_conn, table_name, date_from, date_to):
        schema_path = path.join(settings.BASE_DIR, "_config/db_postgresql/migrations/0001_api_database_model.sql")

        with \
            db_conn.cursor() as cursor, \
            open(schema_path, 'r') as file \
        :
            lines = file.readlines()
            content = ""
            for line in lines:
                line = line.strip()
                if not line.startswith("--"):
                    content = content + line + "\n"

            content = content.replace("#DATE#", table_name)

            # create table
            queries = content.split(';')
            for query in queries:
                if len(query.strip()) > 0:
                    cursor.execute(query)

            # create first table space
            cursor.execute("SELECT id FROM tblspace ORDER BY id LIMIT 1")
            tblspace = cursor.fetchone()
            if tblspace is None:
                tblspace_id = 1  # TODO MM
                cursor.execute("INSERT INTO tblspace (id, name, partition) VALUES (" + str(tblspace_id) + ", 'main table space', 'main table space')")
            else:
                tblspace_id = tblspace[0]

            table_name_full = "DATA_TS" + str(tblspace_id) + "_" + table_name

            # create data tbl row
            cursor.execute("SELECT count(*) FROM datatbl")
            datatbl_count = cursor.fetchone()
            datatbl_id = datatbl_count[0] + 1

            cursor.execute("INSERT INTO datatbl (id, date_from, date_to, name, tblspace_id) VALUES " \
                + "(" + str(datatbl_id) + ", '" + date_from + "', '" + date_to + "', '" + table_name_full + "', " + str(tblspace_id) + ")")

            # create node
            cursor.execute("SELECT count(*) FROM node")
            node_count = cursor.fetchone()
            node_id = node_count[0] + 1

            cursor.execute("INSERT INTO node (id, name, description, connection) VALUES " \
                + "(" + str(node_id) + ", '" + str(node_id) + "', '', '')")
