# -*- coding: utf-8 -*-
from django.core.management.base import BaseCommand, CommandError
from django.db import connections
from django.conf import settings

from os import path, listdir

import importlib
import uuid


class Command(BaseCommand):

    def add_arguments(self, parser):
        parser.add_argument(
            '--database',
            type=str,
            required=True,
            help='Database Type'
        )

    def handle(self, *args, **options):
        db_conn = self._get_db_conn(options)
        self._generate_schema(db_conn, options)

        return str(True)

    def _get_db_conn(self, options):
        database = options['database']

        impmodule = importlib.import_module("_config.db_" + database + ".settings")
        DATABASES = impmodule.DATABASES

        connections.databases[database] = DATABASES['default']
        db_conn = connections[database]

        return db_conn

    def _generate_schema(self, db_conn, options):
        schema_path = path.join(settings.BASE_DIR, "_config/db_postgresql/migrations/0001_api_database.sql")

        with \
            db_conn.cursor() as cursor, \
            open(schema_path, 'r') as file \
        :
            lines = file.readlines()
            content = ""
            for line in lines:
                line = line.strip()
                if not line.startswith("--"):
                    content = content + line + "\n"

            queries = content.split(';')
            for query in queries:
                if len(query.strip()) > 0:
                    cursor.execute(query)
