# -*- coding: utf-8 -*-
from django.core.management.base import BaseCommand, CommandError
from django.conf import settings

from os import path, remove


class Command(BaseCommand):

    def add_arguments(self, parser):
        parser.add_argument(
            '--archives_root_dir',
            type=str,
            required=True,
            help='Root Dir of REX archives'
        )

        parser.add_argument(
            '--route_api',
            type=str,
            required=False,
            help='Route'
        )

        parser.add_argument(
            '--force',
            type=bool,
            required=False,
            help='Replace existing configuration'
        )

    def handle(self, *args, **options):
        archives_root_dir = options['archives_root_dir']
        force = options['force']

        default_settings_path = path.join(settings.BASE_DIR, "_config/impl_rex_archives/settings.py")
        if path.exists(default_settings_path):
            if not force:
                raise CommandError('Application is already initialized, force flag is not enabled!')
            else:
                remove(default_settings_path)

        template_settings_path = path.join(settings.BASE_DIR, "_config/impl_rex_archives/settings_template.py")
        if not path.exists(template_settings_path):
            raise CommandError('Template settings does not exists!')

        with open(template_settings_path, 'r') as file:
            content = file.read()

            if 'route_api' in options and options['route_api']:
                route_api = options['route_api']
                lines = content.splitlines()
                content = ""

                for line in lines:
                    if "# Application definition" in line:
                        content = content + "\n\n" + "# Overrides by bootstrap"
                        content = content + "\n"
                        content = content + "APP_URL_API_ROUTE = \"" + route_api + "\""
                        content = content + "\n"
                        content = content + "DEBUG = False"
                        content = content + "\n"
                        content = content + "\n"
                        content = content + "\n"

                    content = content + line
                    content = content + "\n"

            content = content.replace('#ARCHIVES_ROOT_DIR#', archives_root_dir)

            with open(default_settings_path, "w") as configuration:
                configuration.write(content)

        return str(True)
